@@ -7,7 +7,6 @@ for which a new license (GPL+exception) is in place.
 #ifndef NODEPALETTE_H
 #define NODEPALETTE_H
 
-#include <qbuttongroup.h>
 #include <qpushbutton.h>
 #include <qtoolbutton.h>
 #include <qlabel.h>
@@ -31,7 +30,6 @@ public:
 	NodePalette( QWidget* parent);
 	~NodePalette() {};
 
-	QButtonGroup* ButtonGroup1;
 	QToolButton* MoveNode;
 	QToolButton* MoveControl;
 	QToolButton* AddNode;
@@ -64,6 +62,7 @@ public:
 	QSpinBox *scalePercentage;
 	MSpinBox *scaleDistance;
 	QPushButton* ResetCont;
+	QPushButton* ResetContClip;
 	QPushButton* editEditButton;
 	void setDoc(ScribusDoc *dc, ScribusView *vi);
 	ScribusDoc* currentDocument() const;
@@ -81,6 +80,7 @@ private slots:
 	void ResetControl();
 	void Reset1Control();
 	void ResetContour();
+	void ResetContourToImageClip();
 	void CloseBezier();
 	void SplitPoly();
 	void MirrorH();
@@ -106,14 +106,15 @@ public slots:
 	void PolyStatus(int typ, uint size);
 	void languageChange();
 	void unitChange();
-	void EndEdit(); // allow remote closing
+	void EndEdit(); // allow remote close
 
 protected:
 	void connectSignals();
 	void disconnectSignals();
 	QVBoxLayout* NodePaletteLayout;
-	QGridLayout* ButtonGroup1Layout;
-	QGridLayout* Layout2;
+	QGridLayout *gridLayout;
+	QGridLayout *gridLayout1;
+	QGridLayout *gridLayout2;
 	
 	double unitRatio;
 	
