@@ -61,6 +61,7 @@ for which a new license (GPL+exception) is in place.
 #include "scribusview.h"
 #include "scribusdoc.h"
 #include "scribusstructs.h"
+#include "customfdialog.h"
 
 class ActionManager;
 class AlignDistributePalette;
@@ -68,8 +69,10 @@ class Autoforms;
 class Biblio;
 class BookPalette;
 class CheckDocument;
-class FileWatcher;
+
 class FontCombo;
+class GuideManager;
+class CharSelect;
 class ColorCombo;
 class LayerPalette;
 class LineFormate;
@@ -79,17 +82,18 @@ class MenuManager;
 class NodePalette;
 class PageItem;
 class PagePalette;
-class PluginManager;
+class Preferences;
 class PrefsManager;
 class PrefsContext;
 class PSLib;
+class ReformDoc;
 class ScrAction;
+class ScribusCore;
 class ScribusMainWindow;
 class ScribusQApp;
 class ScribusWin;
 class ScToolBar;
 class SimpleState;
-class SplashScreen;
 class StilFormate;
 class StoryEditor;
 class StyleManager;
@@ -98,11 +102,10 @@ class Tree;
 class UndoManager;
 class UndoPalette;
 class UndoState;
-class WerkToolB;
-class WerkToolBP;
+class ModeToolBar;
+class PDFToolBar;
 
 extern SCRIBUS_API ScribusQApp* ScQApp;
-extern SCRIBUS_API ScribusMainWindow* ScMW;
 
 /**
   * \brief This Class is the base class for your application. It sets up the main
@@ -122,21 +125,15 @@ public:
 	/*!
 	* \retval 0 - ok, 1 - no fonts, ...
 	*/
-	int initScribus(bool showSplash, bool showFontInfo, const QString newGuiLanguage, const QString prefsUserFile);
-	void showSplash(bool);
-	bool splashShowing() const;
-	void closeSplash();
-	void setSplashStatus(const QString&);
-	const QString getGuiLanguage();
+	int initScMW(bool primaryMainwWindow);
 	bool warningVersion(QWidget *parent);
 	void SetShortCut();
 	void startUpDialog();
 	void setDefaultPrinter(const QString&, const QString&, const QString&);
 	void getDefaultPrinter(QString*, QString*, QString*);
-	const bool fileWatcherActive();
 
-	bool doFileNew(double width, double h, double tpr, double lr, double rr, double br, double ab, double sp,
-					bool atf, int fp, int einh, int firstleft, int Ori, int SNr, const QString&, int pageCount=1, bool showView=true);
+	ScribusDoc *doFileNew(double width, double height, double topMargin, double leftMargin, double rightMargin, double bottomMargin, double columnDistance, double columnCount, bool autoTextFrames, int pageArrangement, int unitIndex, int firstPageLocation, int orientation, int firstPageNumber, const QString& defaultPageSize, bool requiresGUI, int pageCount=1, bool showView=true);
+	ScribusDoc *newDoc(double width, double height, double topMargin, double leftMargin, double rightMargin, double bottomMargin, double columnDistance, double columnCount, bool autoTextFrames, int pageArrangement, int unitIndex, int firstPageLocation, int orientation, int firstPageNumber, const QString& defaultPageSize, bool requiresGUI, int pageCount=1, bool showView=true);
 	bool DoFileSave(QString fn);
 	void closeEvent(QCloseEvent *ce);
 	void keyPressEvent(QKeyEvent *k);
@@ -153,17 +150,14 @@ public:
 					  const QMap<int,QPixmap> & thumbs);
 	bool DoSaveAsEps(QString fn);
 	QString CFileDialog(QString wDir = ".", QString caption = "", QString filter = "", QString defNa = "",
-						bool Pre = false, bool mod = true, bool comp = false, bool cod = false,
-						bool onlyDirs = false, bool *docom = 0, bool *doFont = 0);
-	void GetCMSProfiles();
-	void GetCMSProfilesDir(QString pfad);
+						int optionFlags = fdExistingFiles, bool *docom = 0, bool *doFont = 0, bool *doProfiles = 0);
 	/*! \brief Recalculate the colors after changing CMS settings.
 	Call the appropriate document function and then update the GUI elements.
 	\param dia optional progress widget */
 	void recalcColors(QProgressBar *dia = 0);
 	void SwitchWin();
 	void RestoreBookMarks();
-	void ReorgFonts();
+//	void ReorgFonts();
 
 	void emergencySave();
 
@@ -178,8 +172,8 @@ public:
 	 */
 	bool isObjectSpecificUndo();
 	void restore(UndoState* state, bool isUndo);
-	void restoreGroupping(SimpleState *state, bool isUndo);
-	void restoreUngroupping(SimpleState *state, bool isUndo);
+	void restoreGrouping(SimpleState *state, bool isUndo);
+	void restoreUngrouping(SimpleState *state, bool isUndo);
 	void restoreAddPage(SimpleState *state, bool isUndo);
 	void restoreDeletePage(SimpleState *state, bool isUndo);
 	struct CopyPasteBuffer Buffer;
@@ -187,13 +181,6 @@ public:
 	QString Buffer2;
 	QString Buffer3;
 	bool BuFromApp;
-	ProfilesL InputProfiles;
-	ProfilesL InputProfilesCMYK;
-	ProfilesL MonitorProfiles;
-	ProfilesL PrinterProfiles;
-	ProfilesL PDFXProfiles;
-	double DispX;
-	double DispY;
 	int HaveDoc;
 	PrefsContext* dirs;
 	/** \brief view is the main widget which represents your working area. The View
@@ -206,14 +193,15 @@ public:
 	 * information such as filename and does the serialization of your files.
 	 */
 	ScribusDoc *doc;
-    /** \brief the splash screen */
-	SplashScreen *splashScreen;
-	QLabel* mainWindowStatusLabel;
+
+
 	QProgressBar* mainWindowProgressBar;
 	QLabel* mainWindowXPosLabel;
 	QLabel* mainWindowXPosDataLabel;
 	QLabel* mainWindowYPosLabel;
 	QLabel* mainWindowYPosDataLabel;
+	GuideManager *guidePalette;
+	CharSelect *charPalette;
 	Mpalette *propertiesPalette;
 	NodePalette *nodePalette;
 	Tree *outlinePalette;
@@ -230,51 +218,49 @@ public:
 	QMap<QString,QString> Sprachen;
 	QWorkspace *wsp;
 	ScribusWin* ActWin;
-	QString PrefsPfad;
 	QClipboard *ClipB;
 	QString LoadEnc;
-	bool singleClose;
 	bool ScriptRunning;
 	Autoforms* SCustom;
-	WerkToolB* mainToolBar;
-	int HavePngAlpha;
-	int HaveTiffSep;
+	ModeToolBar* mainToolBar;
 
 	QMap<QString, QStringList> InstLang;
 	QMap<QString,QString> LangTransl;
-	FileWatcher* fileWatcher;
+
 	QProcess *ExternalApp;
 
 	QMap<QString, QGuardedPtr<ScrAction> > scrActions;
 	QMap<QString, QGuardedPtr<ScrAction> > scrRecentFileActions;
 	QMap<QString, QGuardedPtr<ScrAction> > scrWindowsActions;
 	QMap<QString, QGuardedPtr<ScrAction> > scrLayersActions;
+	QMap<QString, QGuardedPtr<ScrAction> > scrRecentPasteActions;
 	QDict<QActionGroup> scrActionGroups;
 	MenuManager* scrMenuMgr;
 	ActionManager* actionManager;
-	PluginManager* pluginManager;
 	QStringList RecentDocs;
 
 public slots:
 	void languageChange();
 	void specialActionKeyEvent(const QString& actionName, int unicodevalue);
 	void newView();
+	void ToggleStickyTools();
 	void ToggleAllGuides();
 	void ToggleAllPalettes();
 	void slotStoryEditor();
 	void slotCharSelect();
 	void ImageEffects();
-	QString Collect(const bool compress = false, const bool withFonts = false, const QString& newDirectory=QString::null);
+	QString Collect(const bool compress = false, const bool withFonts = false, const bool withProfiles = false, const QString& newDirectory=QString::null);
 	void AddBookMark(PageItem *ite);
 	void DelBookMark(PageItem *ite);
 	void BookMarkTxT(PageItem *ite);
 	void StoreBookmarks();
-	void ManageGuides();
 	//void setItemFillTransparency(double t);
 	//void setItemLineTransparency(double t);
-	void setMousePositionOnStatusBar(double xp, double yp);
+	void setStatusBarMousePosition(double xp, double yp);
+	void setStatusBarInfoText(QString newText);
 	bool DoFileClose();
 	//bool DoSaveClose();
+	void fontMenuAboutToShow();
 	void windowsMenuAboutToShow();
 	void newActWin(QWidget *w);
 	void closeActiveWindowMasterPageEditor();
@@ -287,6 +273,7 @@ public slots:
 	void slotTest();
 	void slotTest2();
 	void PutScrap();
+	void PutToPatterns();
 	void changeLayer(int);
 	void showLayer();
 	//void LayerRemove(int l, bool dl = false);
@@ -299,6 +286,7 @@ public slots:
 	bool loadPage(QString fileName, int Nr, bool Mpa, const QString& renamedPageName=QString::null);
 
 	void slotGetContent();
+	void slotGetContent2(); // kk2006
 	/*!
 	\author Franz Schmid
 	\brief Appends a Textfile to the Text in the selected Textframe at the Cursorposition
@@ -308,6 +296,8 @@ public slots:
 	void removeRecent(QString fn);
 	void loadRecent(QString fn);
 	void rebuildRecentFileMenu();
+	void rebuildRecentPasteMenu();
+	void pasteRecent(QString fn);
 	void rebuildLayersList();
 	bool slotDocOpen();
 	bool loadDoc(QString);
@@ -334,7 +324,7 @@ public slots:
 	\param options PrintOptions struct to control all settings
 	\sa ScribusMainWindow::slotFilePrint()
 	\retval bool True for success */
-	bool doPrint(PrintOptions *options);
+	bool doPrint(PrintOptions &options);
 	/** \brief exits the application */
 	void slotFileQuit();
 	/** \brief put the marked text/object into the clipboard and remove
@@ -387,7 +377,9 @@ public slots:
 	void slotZoom(double zoomFactor); // 20, 50, 100, or -100 for Fit
 	/** \brief Schaltet Raender ein/aus */
 	void ToggleMarks();
+	void ToggleBleeds();
 	void ToggleFrames();
+	void ToggleLayerMarkers();
 	void ToggleTextLinks();
 	void ToggleTextControls();
 	void ToggleColumnBorders();
@@ -432,7 +424,7 @@ public slots:
 	void setItemFont(int id);
 	void setItemFont2(int id);
 	/** Korrigiert das FontMenu */
-	void AdjustFontMenu(QString nf);
+	void AdjustFontMenu(const QString& nf);
 	void SetNewFont(const QString& nf);
 	/** Setz die Zeichensatzgroesse */
 	void setItemFSize(int id);
@@ -440,7 +432,7 @@ public slots:
 	/** Farbeditor */
 	void slotEditColors();
 	/** Style Manager */
-	void slotStyleManager();
+// 	void slotStyleManager();
 	/*
 	// Setzt den Pen-Tonwert
 	void setPenShade(int sh);
@@ -455,9 +447,9 @@ public slots:
 	// Setzt die Brush-Farbe
 	void setBrushFarbe(QString farbe);
 	*/
-	void setCSMenu(QString f, QString l, int fs, int ls);
+	void setCSMenu();
 	/** Fragt nach den Farben */
-	void GetBrushPen();
+//	void GetBrushPen();
 	/** Erzeugt einen Rahmen */
 	void MakeFrame(int f, int c, double *vals);
 	//** Loescht ein Element */
@@ -466,18 +458,20 @@ public slots:
 	void ObjektDup();
 	/** Dupliziert das Element mehrfach*/
 	void ObjektDupM();
-	/** Reformatiert das Dokument */
+	/** \brief Refromat the document when user click "OK" in ReformDoc dialog.
+	See docSetup() for more info. */
 	bool slotDocSetup();
 	void objectAttributes();
 	void getImageInfo();
 	void generateTableOfContents();
 	void buildFontMenu();
+	/*! \brief Change Preferences dialog.
+	See prefsOrg for more info. It's very similar to docSetup/slotDocSetup. */
 	void slotPrefsOrg();
-	void slotEditStyles();
-	void saveStyles(StilFormate *dia);
-	void slotEditLineStyles();
-	void saveLStyles(LineFormate *dia);
-	void setNewAbStyle(int a);
+	void saveStyles(StilFormate *dia); //still required for style save from SE
+	void setNewAlignment(int a);
+	void setNewParStyle(const QString& name);
+	void setNewCharStyle(const QString& name);
 	void setAbsValue(int a);
 	void selectItemsFromOutlines(PageItem *ite);
 	void selectItemsFromOutlines(int Page, int Item, bool single = false);
@@ -500,7 +494,7 @@ public slots:
 	 * @todo Make this work with real page numbers, negative numbers and document sections when they are implemented
 	*/
 	void ApplyMasterPage();
-	void Apply_MasterPage(QString in, int Snr, bool reb = true);
+	void Apply_MasterPage(QString pageName, int pageNumber, bool reb = true);
 	void GroupObj(bool showLockDia = true);
 	void UnGroupObj();
 	void StatusPic();
@@ -515,16 +509,35 @@ public slots:
 	/*! \brief call gimp and wait upon completion */
 	void callImageEditor();
 	void docCheckToggle(bool visible);
-	void scanDocument();
+	//! \brief Scan a document for errors, return true on errors found
+	bool scanDocument();
 	void setUndoMode(bool isObjectSpecific);
 	//! \brief Apply a Lorem Ipsum to the each item in a selection
 	void insertSampleText();
 	//void sendToLayer(int layerNumber);
 	void updateItemLayerList();
+	/*! \brief Apply changes from ReformDoc dialog.
+	It's called from this->slotDocSetup() or from ReformDoc directly.
+	\param dia a reference to the ReformDoc dialog */
+	void docSetup(ReformDoc* dia);
+	/*! \brief Apply changes from Preferences dialog.
+	It's called from this->slotPrefsOrg() or from Preferences directly.
+	\param dia a reference to the Preferences dialog */
+	void prefsOrg(Preferences* dia);
+	//! \brief Insert a frame friendly dialog
+	void slotInsertFrame();
+	//! \brief manages the documents patterns
+	void managePatterns();
+	//! \brief enable or disable the unicode actions and their menus
+	void enableTextActions(QMap<QString, QGuardedPtr<ScrAction> > *actionMap, bool enabled, const QString& fontName=QString::null);
+	//! \brief allow SE to get the SM for edit stlyes
+	StyleManager *styleMgr() const {return styleManager;};
 
 signals:
+	void TextStyle(const ParagraphStyle&);
+//deprecated: (av)
 	void TextISize(int);
-	void TextIFont(QString);
+	void TextIFont(const QString&);
 	void TextUSval(int);
 	void TextStil(int);
 	void TextFarben(QString, QString, int, int);
@@ -536,8 +549,6 @@ signals:
 	void TextUnderline(int, int);
 	void TextStrike(int, int);
 
-	void prefsChanged();
-
 protected:
 	/*!
 	\brief Receive key events from palettes such as palette hiding events. Possibly easier way but this is cleaner than before. No need to modify all those palettes and each new one in future.
@@ -554,20 +565,19 @@ private:
 	void initStatusBar(); // setup the statusbar
 	void initToolBars(); // setup the toolbars
 	//Returns false when there are no fonts
-	const bool initFonts(const bool showFontInfo);
 	void initHyphenator();
 	void initDefaultValues();
 	void initKeyboardShortcuts();
 	void initPalettes();
 	void initScrapbook();
-	void initCMS();
+
 	void updateColorMenu(QProgressBar* progressBar=NULL);
 
-	QString guiLanguage;
+	QLabel* mainWindowStatusLabel;
 	QString recentFileMenuName;
+	QString recentPasteMenuName;
 	QString layerMenuName;
 	QPixmap noIcon;
-	bool scribusInitialized;
 	ColorCombo *ColorMenC;
 	/** ShapeMenu enthaelt die Rahmenformen */
 	QPopupMenu *ShapeMenu;
@@ -576,7 +586,7 @@ private:
 	FontCombo* FontSub;
 	ScToolBar *fileToolBar;
 	ScToolBar *editToolBar;
-	WerkToolBP* pdfToolBar;
+	PDFToolBar* pdfToolBar;
 	QToolButton* DatOpe;
 	QToolButton* DatSav;
 	QToolButton* DatClo;
@@ -596,7 +606,7 @@ private:
 	int viewUndoPalette;
 
 	bool palettesStatus[10];
-	bool guidesStatus[10];
+	bool guidesStatus[13];
 
 	bool keyrep;
 	/** @brief Tells if an arrow key is pressed down */
@@ -606,11 +616,9 @@ private:
 
 	void addNewPages(int wo, int where, int numPages, double height, double width, int orient, QString siz, bool mov, QStringList* basedOn = 0);
 	QMap<int,QString> FontID;
-	int HaveGS;
+
 	void *PSDriver;
 	int DocNr;
-	UndoManager *undoManager;
-	PrefsManager *prefsManager;
 	bool PrinterUsed;
 	struct PDe {
 					QString Pname;
@@ -624,7 +632,9 @@ private:
 	int storedViewYCoor;
 	double storedViewScale;
 	StyleManager *styleManager;
+	UndoManager *undoManager;
+	PrefsManager *prefsManager;
+	QString currentFontForFontMenu;
 };
 
 #endif
-
