@@ -39,8 +39,10 @@ for which a new license (GPL+exception) is in place.
 #include <qstringlist.h>
 #include <qtimer.h>
 
+#include "gtgettext.h" //CB For the ImportSetup struct and itemadduserframe
 #include "scribusapi.h"
 #include "scribusstructs.h"
+#include "pagesize.h"
 #include "prefsstructs.h"
 #include "documentinformation.h"
 #include "undoobject.h"
@@ -49,11 +51,12 @@ for which a new license (GPL+exception) is in place.
 #include "pageitem_line.h"
 #include "pageitem_textframe.h"
 #include "pagestructs.h"
+#include "usertaskstructs.h"
+#include "styles/styleset.h"
+#include "scpattern.h"
+#include "scguardedptr.h"
 
-
-#ifdef HAVE_CMS
-	#include CMS_INC
-#endif
+#include CMS_INC
 
 #include <ft2build.h>
 #include FT_FREETYPE_H
@@ -64,6 +67,8 @@ class PDFOptions;
 class Hyphenator;
 class Selection;
 class ScribusView;
+class ScribusMainWindow;
+class ResourceCollection;
 
 class QProgressBar;
 
@@ -76,7 +81,9 @@ class SCRIBUS_API ScribusDoc : public QObject, public UndoObject
 
 public:
 	ScribusDoc();
+	ScribusDoc(const QString& docName, int unitIndex, const PageSize& pagesize, const MarginStruct& margins, const DocPagesSetup& pagesSetup);
 	~ScribusDoc();
+	void init();
 	void setup(const int, const int, const int, const int, const int, const QString&, const QString&);
 	void setLoading(const bool);
 	bool isLoading() const;
@@ -90,19 +97,27 @@ public:
 	 * @brief Return the view associated with the document
 	 */
 	ScribusView* view() const;
+	ScribusMainWindow* scMW() const {return m_ScMW;}
+	void setGUI(bool hasgui, ScribusMainWindow* mw, ScribusView* view);
+
+	/**
+	 * @brief Return the guarded object associated with the document
+	 */
+	const ScGuardedPtr<ScribusDoc>& guardedPtr() const;
 	
 	// Add, delete and move pages
 	
 	Page* addPage(const int pageNumber, const QString& masterPageName=QString::null, const bool addAutoFrame=false);
+	void deletePage(const int);
+	//! @brief Add a master page with this function, do not use addPage
+	Page* addMasterPage(const int, const QString&);
 	void deleteMasterPage(const int);
 	//! @brief Rename a master page
 	bool renameMasterPage(const QString& oldPageName, const QString& newPageName);
-	void deletePage(const int);
-	/**
-	 * @brief Add a master page with this function, do not use addPage
-	 */
-	Page* addMasterPage(const int, const QString&);
-
+	//! @brief Create the default master pages based on the layout selected by the user, ie, Normal, Normal Left, etc.
+	void createDefaultMasterPages();
+	//! @brief Create the requested pages in a new document, run after createDefaultMasterPages()
+	void createNewDocPages(int pageCount);
 	/**
 	 * @brief Add the automatic text frame to the page
 	 * @param pageNumber page number
@@ -140,6 +155,13 @@ public:
 	 * @return Number of the layer created
 	 */
 	int addLayer(const QString& layerName=QString::null, const bool activate=false);
+	/**
+	 * @brief Copies a layer from the current document
+	 * @param layerNumberToCopy source layer
+	 * @param whereToInsert target layer
+	 * @return Success or failure
+	 */
+	void copyLayer(int layerNumberToCopy, int whereToInsert);
 	/**
 	 * @brief Delete a layer from the current document
 	 * @param layerNumber of layer
@@ -209,12 +231,77 @@ public:
 	 * @return Locked or not
 	 */
 	bool layerLocked(const int layerNumber);
+	/**
+	 * @brief Set the layer flow via the layer number
+	 * @param layerNumber Number of the layer
+	 * @param flow true = Text flows around objects on this layer
+	 * @return Success or failure
+	 */
+	bool setLayerFlow(const int layerNumber, const bool flow);
+	/**
+	 * @brief does text flow around objects on this layer
+	 * @param layerNumber Number of the layer
+	 * @return flow or not
+	 */
+	bool layerFlow(const int layerNumber);
+	/**
+	 * @brief Set the layer transparency via the layer number
+	 * @param layerNumber Number of the layer
+	 * @param trans transparency value 0.0 - 1.0
+	 * @return Success or failure
+	 */
+	bool setLayerTransparency(const int layerNumber, double trans);
+	/**
+	 * @brief returns the layer transparency
+	 * @param layerNumber Number of the layer
+	 * @return transparency value 0.0 - 1.0
+	 */
+	double layerTransparency(const int layerNumber);
+	/**
+	 * @brief Set the layer layerBlendMode via the layer number
+	 * @param layerNumber Number of the layer
+	 * @param trans layerBlendMode
+	 * @return Success or failure
+	 */
+	bool setLayerBlendMode(const int layerNumber, int blend);
+	/**
+	 * @brief returns the layer BlendMode
+	 * @param layerNumber Number of the layer
+	 * @return layerBlendMode
+	 */
+	int layerBlendMode(const int layerNumber);
 	/**
 	 * @brief Return the level of the requested layer
 	 * @param layerNumber Number of the layer
 	 * @return Level of the layer
 	 */
 	int layerLevelFromNumber(const int layerNumber);
+	/**
+	 * @brief Set the layer marker color
+	 * @param layerNumber Number of the layer
+	 * @param color color of the marker
+	 * @return Success or failure
+	 */
+	bool setLayerMarker(const int layerNumber, QColor color);
+	/**
+	 * @brief returns the layer marker color
+	 * @param layerNumber Number of the layer
+	 * @return marker color
+	 */
+	QColor layerMarker(const int layerNumber);
+	/**
+	 * @brief Set the layer outline mode via the layer number
+	 * @param layerNumber Number of the layer
+	 * @param outline true = layer is displayed in outlines only
+	 * @return Success or failure
+	 */
+	bool setLayerOutline(const int layerNumber, const bool outline);
+	/**
+	 * @brief is this layer in outline mode
+	 * @param layerNumber Number of the layer
+	 * @return outline or not
+	 */
+	bool layerOutline(const int layerNumber);
 	/**
 	 * @brief Return the number of the layer at a certain level
 	 * @param layerLevel Layer level
@@ -284,10 +371,48 @@ public:
 	//Items
 	bool deleteTaggedItems();
 
-	bool AddFont(QString name, int fsize = 10);
-	bool OpenCMSProfiles(ProfilesL InPo, ProfilesL MoPo, ProfilesL PrPo);
+	/*!
+		* @brief Builds a qmap of the icc profiles used within the document
+	 */
+	void getUsedProfiles(ProfilesL& usedProfiles);
+	bool OpenCMSProfiles(ProfilesL InPo, ProfilesL InPoCMYK, ProfilesL MoPo, ProfilesL PrPo);
 	void CloseCMSProfiles();
-	void loadStylesFromFile(QString fileName, QValueList<ParagraphStyle> *tempStyles = NULL);
+	/**
+	 * @brief Switch Colormanagement on or of
+	 * @param enable bool, if true Colormanagement is switched on, else off
+	 */
+	void enableCMS(bool enable);
+	
+	const ParagraphStyle& paragraphStyle(QString name) { return docParagraphStyles[name]; }
+	const StyleSet<ParagraphStyle>& paragraphStyles()   { return docParagraphStyles; }
+	bool isDefaultStyle( const ParagraphStyle& p ) const { return docParagraphStyles.isDefault(p); }
+	bool isDefaultStyle( const CharStyle& c ) const { return docCharStyles.isDefault(c); }
+// 	bool isDefaultStyle( LineStyle& l ) const { return MLineStyles......; }
+
+	void getNamedResources(ResourceCollection& lists) const;
+	void replaceNamedResources(ResourceCollection& newNames);	
+	
+	void redefineStyles(const StyleSet<ParagraphStyle>& newStyles, bool removeUnused=false);
+	/**
+	 * @brief Remove any reference to old styles and replace with new name. This needs to be
+	 *        called when a style was removed. New name may be "".
+	 * @param newNameForOld a map which maps the name of any style to remove to a new stylename
+	 */
+	void replaceStyles(const QMap<QString,QString>& newNameForOld);
+	void loadStylesFromFile(QString fileName, StyleSet<ParagraphStyle> *tempStyles = NULL,
+	                                          StyleSet<CharStyle> *tempCharStyles = NULL,
+	                                          QMap<QString, multiLine> *tempLineStyles = NULL);
+
+	const CharStyle& charStyle(QString name) { return docCharStyles[name]; }
+	const StyleSet<CharStyle>& charStyles()  { return docCharStyles; }
+	void redefineCharStyles(const StyleSet<CharStyle>& newStyles, bool removeUnused=false);
+	/**
+	 * @brief Remove any reference to old styles and replace with new name. This needs to be
+	 *        called when a style was removed. New name may be "".
+	 * @param newNameForOld a map which maps the name of any style to remove to a new stylename
+	 */
+	void replaceCharStyles(const QMap<QString,QString>& newNameForOld);
+
 	/**
 	 * @brief Should guides be locked or not
 	 * @param isLocked If true guides on pages cannot be moved if false they
@@ -313,14 +438,35 @@ public:
 	 */
 	QStringList getItemAttributeNames();
 
+	bool AddFont(QString name, int fsize = 10);
 	/*!
-	 * @brief Returns a qmap of the fonts used within the document
+	 * @brief Replace line style colors
 	 */
-	void getUsedFonts(QMap<QString,int> *Really);
+	void replaceLineStyleColors(const QMap<QString, QString>& colorMap);
+	/*!
+	 * @brief Returns a qmap of the fonts and  their glyphs used within the document
+	 */
+	void getUsedFonts(QMap<QString,QMap<uint, FPointArray> > &Really);
+	void checkItemForFonts(PageItem *it, QMap<QString, QMap<uint, FPointArray> > & Really, uint lc);
 	/*!
 	* @brief Builds a qmap of the colours used within the document
 	*/
 	void getUsedColors(ColorList &colorsToUse, bool spot = false);
+	/*!
+	* @brief Return if a specific color is used by line styles
+	*/
+	bool lineStylesUseColor(const QString& colorName);
+	/*!
+	* @brief Set the patterns for a document
+	*/
+	bool addPattern(QString &name, ScPattern& pattern);
+	void setPatterns(QMap<QString, ScPattern> &patterns);
+	/*!
+	* @brief Builds a QStringList of the patterns used within the document
+	*/
+	QStringList getUsedPatterns();
+	QStringList getUsedPatternsSelection();
+	QStringList getUsedPatternsHelper(QString pattern, QStringList &results);
 	/*!
 	 * @brief TODO: Reorganise the fonts.. how? Moved from scribus.cpp
 	 */
@@ -334,11 +480,12 @@ public:
 	/**
 	 * @brief Apply a master page
 	 */
-	bool applyMasterPage(const QString& in, const int);
+	bool applyMasterPage(const QString& pageName, const int pageNumber);
 	/**
 	 * @brief Undo function for applying a master page
 	 */
 	void restoreMasterPageApplying(SimpleState *state, bool isUndo);
+	void restorePageCopy(SimpleState *state, bool isUndo);
 	/**
 	 * @brief Save function
 	 */
@@ -386,7 +533,28 @@ public:
 	\param outline outline color name
 	*/
 	int itemAdd(const PageItem::ItemType itemType, const PageItem::ItemFrameType frameType, const double x, const double y, const double b, const double h, const double w, const QString& fill, const QString& outline, const bool itemFinalised);
+
+	/** Add an item to the page based on the x/y position. Item will be fitted to the closest guides/margins */
+	int itemAddArea(const PageItem::ItemType itemType, const PageItem::ItemFrameType frameType, const double x, const double y, const double w, const QString& fill, const QString& outline, const bool itemFinalised);
 	
+	/**
+	 * @brief Allow the user to create a frame easily with some simple placement and sizing options
+	 * @param type Type of frame. @sa PageItem::ItemType
+	 * @param locationType Whether the item is only created on the current page or a range 
+	 * @param positionType Location of frame. 0 for 0,0 of page. 1 for 0,0 of page margins, 2, for custom
+	 * @param sizeType Size of frame. 0 for page size, 1 for margin size, 2 for custom
+	 * @param fX X position
+	 * @param fY Y position
+	 * @param fWidth Width of frame
+	 * @param fHeight Height of frame
+	 * @param source Source image or text file for image or text frames
+	 * @param impsetup Import setup for text frames for gtGetText
+	 * @param columnCount Number of columns if created item is a text frame
+	 * @param columnGap Gap between columns if created item is a text frame
+	 * @return 
+	 */
+	int itemAddUserFrame(InsertAFrameData &iafData);
+
 	/**
 	 * @brief Commit item creation when a user has click-drag created an item
 	 * Only called from ScribusView. Note the undo target is the page, so the undo code remains their for now.
@@ -399,7 +567,8 @@ public:
 	 * Only to be called from itemAdd()
 	 */
 	void itemAddDetails(const PageItem::ItemType itemType, const PageItem::ItemFrameType frameType, const int itemNumber);
-	
+
+	uint getItemNrfromUniqueID(uint unique);
 	//itemDelete
 	//itemBlah...
 	
@@ -407,7 +576,14 @@ public:
 	 * @brief Run this common frame item update code
 	 */
 	void updateFrameItems();
-	
+	/**
+	 * @brief Renumbers the items into the order they are stored in in the lists.
+	 * Utility function used in various places, basically handles keeping items numbered in the way
+	 * they are layered. When layer is a property and not a fuction of storage, this should be removed.
+	 * Depends on the Items pointer pointing to the correct item list (doc, master, etc).
+	 * @sa updateFrameItems();
+	 */
+	void renumberItemsInListOrder();
 	/**
 	 * @brief Doc uses automatic text frames?
 	 */
@@ -452,6 +628,8 @@ public:
 	 */
 	const double getXOffsetForPage(const int);
 	const double getYOffsetForPage(const int);
+	void getBleeds(int pageNumber, double *bleedTop, double *bleedBottom, double *bleedLeft, double *bleedRight);
+	void getBleeds(Page* page, double *bleedTop, double *bleedBottom, double *bleedLeft, double *bleedRight);
 	
 	/**
 	 * @brief Item type conversion functions
@@ -504,10 +682,6 @@ public:
 	 *
 	 */
 	void updateSectionPageNumbersToPages();
-	/**
-	 *
-	 *
-	 */
 	/**
 	 * 
 	 * @param otherPageIndex 
@@ -528,10 +702,11 @@ public:
 	 * @brief Update the fill and line QColors for all items in the doc
 	 */
 	void updateAllItemQColors();
-	/**
-	 * 
-	 */
-	void buildAlignItemList();
+	//! @brief Some internal align tools
+	typedef enum {alignFirst, alignLast, alignPage, alignMargins, alignGuide, alignSelection } AlignTo;
+	void buildAlignItemList(Selection* customSelection=0);
+	bool startAlign();
+	void endAlign();
 	/**
 	 * \brief Insert a color into the documents color list
 	 * @param nam Name of the colour
@@ -553,31 +728,45 @@ public:
 	
 	bool sendItemSelectionToBack();
 	bool bringItemSelectionToFront();
-	
+
+	void itemSelection_ApplyParagraphStyle(const ParagraphStyle & newstyle, Selection* customSelection=0);
+	void itemSelection_SetParagraphStyle(const ParagraphStyle & newstyle, Selection* customSelection=0);
+	void itemSelection_ApplyCharStyle(const CharStyle & newstyle, Selection* customSelection=0);
+	void itemSelection_SetCharStyle(const CharStyle & newstyle, Selection* customSelection=0);
+	void itemSelection_EraseParagraphStyle(Selection* customSelection=0);
+	void itemSelection_EraseCharStyle(Selection* customSelection=0);
+
+	void itemSelection_SetNamedParagraphStyle(const QString & name, Selection* customSelection=0);
+	void itemSelection_SetNamedCharStyle(const QString & name, Selection* customSelection=0);
+
 	void ChLineWidth(double w);
-	void ChLineArt(PenStyle w);
-	void ChLineJoin(PenJoinStyle w);
-	void ChLineEnd(PenCapStyle w);
-	void ChLineSpa(double w);
-	void ChLineSpaMode(int w);
+	void ChLineArt(Qt::PenStyle w);
+	void ChLineJoin(Qt::PenJoinStyle w);
+	void ChLineEnd(Qt::PenCapStyle w);
+	void itemSelection_SetAlignment(int w, Selection* customSelection=0);
+	void itemSelection_SetLineSpacing(double w, Selection* customSelection=0);
+	void itemSelection_SetLineSpacingMode(int w, Selection* customSelection=0);
 	//void ChLocalXY(double x, double y);
 	//void ChLocalSc(double x, double y);
-	void ItemFont(QString fon);
-	void ItemTextBrush(QString farbe);
-	void ItemTextBrushS(int sha);
-	void ItemTextPen(QString farbe);
-	void ItemTextPenS(int sha);
-	void ItemTextScaleV(int sha);
-	void ItemTextScale(int sha);
-	void setItemTextBase(int sha);
-	void setItemTextOutline(int sha);
-	void setItemTextShadow(int shx, int shy);
-	void setItemTextUnderline(int pos, int wid);
-	void setItemTextStrike(int pos, int wid);
-	void chTyStyle(int s);
-	void chAbStyle(PageItem *currItem, int s);
-	void chKerning(int us);
-	void chFSize(int size);
+	void itemSelection_SetFont(QString fon, Selection* customSelection=0);
+	void itemSelection_SetFillColor(QString farbe, Selection* customSelection=0);
+	void itemSelection_SetFillShade(int sha, Selection* customSelection=0);
+	void itemSelection_SetStrokeColor(QString farbe, Selection* customSelection=0);
+	void itemSelection_SetStrokeShade(int sha, Selection* customSelection=0);
+	void itemSelection_SetScaleV(int, Selection* customSelection=0);
+	void itemSelection_SetScaleH(int, Selection* customSelection=0);
+	void itemSelection_SetBaselineOffset(int, Selection* customSelection=0);
+	void itemSelection_SetOutlineWidth(int, Selection* customSelection=0);
+	void itemSelection_SetShadowOffsets(int shx, int shy, Selection* customSelection=0);
+	void itemSelection_SetUnderline(int pos, int wid, Selection* customSelection=0);
+	void itemSelection_SetStrikethru(int pos, int wid, Selection* customSelection=0);
+	void itemSelection_SetEffects(int s, Selection* customSelection=0);
+	void itemSelection_SetOpticalMargins(int i, Selection* customSelection=0);
+	
+//	void chAbStyle(PageItem *currItem, int s);
+
+	void itemSelection_SetTracking(int us, Selection* customSelection=0);
+	void itemSelection_SetFontSize(int size, Selection* customSelection=0);
 	//void FlipImageH();
 	//void FlipImageV();
 	void MirrorPolyH(PageItem *currItem);
@@ -585,13 +774,44 @@ public:
 	
 	void setRedrawBounding(PageItem *currItem);
 	void adjustCanvas(FPoint minPos, FPoint maxPos, bool absolute = false);
-	void recalcPicturesRes();
+	void recalcPicturesRes(bool applyNewRes = false);
 	void connectDocSignals();
 	void removeLayer(int l, bool dl = false); //FIXME: Make protected once scripter function no longer uses this directly
+	/*! \brief We call changed() whenever the document needs to know it has been changed.
+	 *  If the document is the primary document in a main window, it will signal to enable/disable
+	 * certain operations.
+	 */
 	void changed();
+	/*! \brief Get pointer to the current page
+	\retval Page* current page object */
+	Page* currentPage();
+	/*! \brief Set new current page
+	\param newPage New current page */
+	void setCurrentPage(Page *newPage);
+	bool hasGUI() const {return m_hasGUI;}
+	/*! \brief Apply grid to a QPoint, from ScribusView */
+	QPoint ApplyGrid(const QPoint& in);
+	/*! \brief Apply grid to an FPoint, from ScribusView */
+	FPoint ApplyGridF(const FPoint& in);
+	/*! \brief Does this doc have any TOC setups and potentially a TOC to generate */
+	bool hasTOCSetup() { return !docToCSetups.empty(); }
+	//! \brief Get the closest guide to the given point
+	void getClosestGuides(double xin, double yin, double *xout, double *yout);
+	//! \brief Snap an item to the guides
+	void SnapToGuides(PageItem *currItem);
+	bool ApplyGuides(double *x, double *y);
+	bool MoveItem(double newX, double newY, PageItem* ite, bool fromMP = false);
+	void RotateItem(double win, int ite);
+	void RotateItem(double win, PageItem *currItem);
+	void MoveRotated(PageItem *currItem, FPoint npv, bool fromMP = false);
+	bool SizeItem(double newX, double newY, int ite, bool fromMP = false, bool DoUpdateClip = true, bool redraw = true);
+	bool SizeItem(double newX, double newY, PageItem *pi, bool fromMP = false, bool DoUpdateClip = true, bool redraw = true);
+	bool MoveSizeItem(FPoint newX, FPoint newY, int ite, bool fromMP = false, bool constrainRotation=false);
+	void AdjustItemSize(PageItem *currItem);
 	
 protected:
 	void addSymbols();
+	bool m_hasGUI;
 	ApplicationPrefs& prefsData;
 	UndoManager * const undoManager;
 	bool loading;
@@ -602,6 +822,9 @@ protected:
 	bool automaticTextFrames; // Flag for automatic Textframes
 	bool m_masterPageMode;
 	QMap<QString, double> m_constants;
+	ScribusMainWindow* m_ScMW;
+	ScribusView* m_View;
+	ScGuardedObject<ScribusDoc> m_guardedObject;
 	
 public: // Public attributes
 	bool is12doc; //public for now, it will be removed later
@@ -612,56 +835,61 @@ public: // Public attributes
 	int viewID;
 	bool SnapGuides;
 	bool GuideLock;
-	/** Scratch space around Pages */
-	double ScratchLeft;
-	double ScratchRight;
-	double ScratchTop;
-	double ScratchBottom;
-/** Minimum and Maximum Points of Document */
+	/** \brief Scratch space around Pages */
+	MarginStruct scratch;
+// 	double ScratchLeft;
+// 	double ScratchRight;
+// 	double ScratchTop;
+// 	double ScratchBottom;
+	/** \brief Minimum and Maximum Points of Document */
 	FPoint minCanvasCoordinate;
 	FPoint maxCanvasCoordinate;
 	double rulerXoffset;
 	double rulerYoffset;
-  /** List of Pages */
+	/** \brief List of Pages */
 	QPtrList<Page>* Pages;
-  /** List of Master Pages */
+	/** \brief List of Master Pages */
 	QPtrList<Page> MasterPages;
-  /** List of Document Pages */
+	/** \brief List of Document Pages */
 	QPtrList<Page> DocPages;
-  /** Mapping Master Page Name to Master Page numbers */
+	/** \brief Mapping Master Page Name to Master Page numbers */
 	QMap<QString,int> MasterNames;
-  /** List of Objects */
+	/** \brief List of Objects */
 	QPtrList<PageItem>* Items;
 	QPtrList<PageItem> MasterItems;
 	QPtrList<PageItem> DocItems;
 	QPtrList<PageItem> FrameItems;
 	Selection* const m_Selection;
-  /** Pagewidth  */
+	/** \brief Pagewidth  */
 	double pageWidth;
-  /** Pageheight */
+	/** \brief Pageheight */
 	double pageHeight;
-  /** Number of Pages */
+	/* Number of Pages */
 	// int pageCount; Disabled CR no longer required
-  /** Margins */
+	/** \brief Margins */
 	MarginStruct pageMargins;
 	QValueList<PageSet> pageSets;
-  /** Number of Columns */
+	MarginStruct bleeds;
+// 	double BleedTop;
+// 	double BleedLeft;
+// 	double BleedRight;
+// 	double BleedBottom;
+	/** \brief Number of Columns */
 	double PageSp;
-  /** Distance of Columns */
+	/** \brief Distance of Columns */
 	double PageSpa;
- /** current Pagelayout */
+	/** \brief current Pagelayout */
 	int currentPageLayout;
-	/** Flag fuer Hoch- oder Querformat 0 = Hochformat */
+	/** \brief Flag fuer Hoch- oder Querformat 0 = Hochformat */
 	int PageOri;
-	QString PageSize;
-	/** Erste Seitennummer im Dokument */
+	QString m_pageSize;
+	/** \brief Erste Seitennummer im Dokument */
 	int FirstPnum;
-  /** Flag fuer Rasterbenutzung */
+	/** \brief Flag fuer Rasterbenutzung */
 	bool useRaster;
-  /** Im Dokument benutzte Farben */
+	/** \brief Im Dokument benutzte Farben */
 	ColorList PageColors;
-	Page* currentPage;
-  /** InfoStrings fuer das aktuelle Dokument */
+	/** \brief InfoStrings fuer das aktuelle Dokument */
 	DocumentInformation documentInfo;
 	int appMode;
 	int SubMode;
@@ -673,24 +901,8 @@ public: // Public attributes
 	QValueList<AlignObjs> AObjects;
 	QColor papColor;
 	int CurrentSel;
-	int CurrentStyle;
-	int currentParaStyle;
-	QString CurrFont;
-	int CurrFontSize;
-	QString CurrTextFill;
-	int CurrTextFillSh;
-	QString CurrTextStroke;
-	int CurrTextStrokeSh;
-	int CurrTextScale;
-	int CurrTextScaleV;
-	int CurrTextBase;
-	int CurrTextShadowX;
-	int CurrTextShadowY;
-	int CurrTextOutline;
-	int CurrTextUnderPos;
-	int CurrTextUnderWidth;
-	int CurrTextStrikePos;
-	int CurrTextStrikeWidth;
+	ParagraphStyle currentStyle;
+
 	bool EditClip;
 	int EditClipMode;
 	typoPrefs typographicSettings;
@@ -698,39 +910,44 @@ public: // Public attributes
 	toolPrefs toolSettings;
 	QMap<QString, checkerPrefs> checkerProfiles;
 	QString curCheckProfile;
-  /** Letztes Element fuer AutoTextrahmen */
+	/** \brief Letztes Element fuer AutoTextrahmen */
 	PageItem *LastAuto;
-  /** Erstes Element fuer AutoTextrahmen */
+	/** \brief Erstes Element fuer AutoTextrahmen */
 	PageItem *FirstAuto;
 	bool DragP;
 	bool leaveDrag;
 	PageItem *DraggedElem;
 	PageItem *ElemToLink;
 	QValueList<uint> DragElements;
-	QValueList<ParagraphStyle> docParagraphStyles;
+private:
+	StyleSet<ParagraphStyle> docParagraphStyles;
+	StyleSet<CharStyle> docCharStyles;
+public:
 	QValueList<Layer> Layers;
 	bool marginColored;
 	int GroupCounter;
 	CMSData CMSSettings;
-#ifdef HAVE_CMS
-	cmsHPROFILE DocInputProf;
+	cmsHPROFILE DocInputImageRGBProf;
+	cmsHPROFILE DocInputImageCMYKProf;
+	cmsHPROFILE DocInputRGBProf;
+	cmsHPROFILE DocInputCMYKProf;
 	cmsHPROFILE DocOutputProf;
 	cmsHPROFILE DocPrinterProf;
-	cmsHTRANSFORM stdTransCMYK2Mon;
-	cmsHTRANSFORM stdTransRGBDoc2Mon;
-	cmsHTRANSFORM stdTransRGBDoc2CMYK;
-	cmsHTRANSFORM stdTransCMYK2RGBDoc;
-	cmsHTRANSFORM stdProofRGB;
-	cmsHTRANSFORM stdProofRGBGC;
-	cmsHTRANSFORM stdProofCMYK;
-	cmsHTRANSFORM stdProofCMYKGC;
+	cmsHTRANSFORM stdTransRGBMon;
+	cmsHTRANSFORM stdTransCMYKMon;
+	cmsHTRANSFORM stdProof;
 	cmsHTRANSFORM stdTransImg;
 	cmsHTRANSFORM stdProofImg;
+	cmsHTRANSFORM stdProofImgCMYK;
+	cmsHTRANSFORM stdTransCMYK;
+	cmsHTRANSFORM stdProofCMYK;
+	cmsHTRANSFORM stdTransRGB;
+	cmsHTRANSFORM stdProofGC;
+	cmsHTRANSFORM stdProofCMYKGC;
 	bool SoftProofing;
 	bool Gamut;
-	int IntentMonitor;
-	int IntentPrinter;
-#endif
+	int IntentColors;
+	int IntentImages;
 	bool HasCMS;
 	QMap<QString,QString> JavaScripts;
 	int TotalItems;
@@ -741,6 +958,7 @@ public: // Public attributes
 	bool AutoCheck;
 	
 	PDFOptions PDF_Options;
+	PrintOptions Print_Options;
 	bool RePos;
 	struct BookMa {
 					QString Title;
@@ -761,14 +979,20 @@ public: // Public attributes
 	bool AutoSave;
 	int AutoSaveTime;
 	QTimer * const autoSaveTimer;
-	FT_Library   library;
-	QMap<QString,FT_Face> FFonts;
 	QMap<QString,multiLine> MLineStyles;
 	QValueList<ArrowDesc> arrowStyles;
+	QMap<QString, ScPattern> docPatterns;
 	QWidget* WinHan;
 	bool DoDrawing;
-	QValueList<int> OpenNodes;
+	struct OpenNodesList
+	{
+		int type;
+		Page *page;
+		PageItem *item;
+	};
+	QValueList<OpenNodesList> OpenNodes;
 	QTimer *CurTimer;
+	QMap<int, errorCodes> docLayerErrors;
 	QMap<int, errorCodes> docItemErrors;
 	QMap<int, errorCodes> masterItemErrors;
 	//Attributes to be applied to frames
@@ -782,9 +1006,10 @@ public: // Public attributes
 	FPointArray symNewCol;
 	FPointArray symNewFrame;
 	
-	Hyphenator * const docHyphenator;
+	Hyphenator * docHyphenator;
 private:
 	bool _itemCreationTransactionStarted;
+	Page* _currentPage;
 	
 signals:
 	//Lets make our doc talk to our GUI rather than confusing all our normal stuff
@@ -793,6 +1018,7 @@ signals:
 	 */
 	void docChanged();
 	void updateContents();
+	void updateContents(const QRect &r);
 	void refreshItem(PageItem *);
 	void canvasAdjusted(double width, double height, double dX, double dY);
 	void firstSelectedItemType(int);
@@ -807,6 +1033,8 @@ signals:
 	 * This also applies to Master Pages
 	 */
 	void signalRebuildOutLinePalette();
+	//! Temporary signal for SizeItem
+	void widthAndHeight(double, double);
 	
 public slots:
 	void itemSelection_ToggleLock();
@@ -816,25 +1044,60 @@ public slots:
 	void itemSelection_ChangePreviewResolution(int id);
 	void itemSelection_ClearItem(Selection* customSelection=0);
 	//! Delete the items in the current selection. When force is true, we do not warn the user and make SE happy too. Force is used from @sa Page::restorePageItemCreation
-	void itemSelection_DeleteItem(Selection* customSelection=0, bool forceDeletion = false);
+	void itemSelection_DeleteItem(Selection* customSelection=0, bool forceDeletion=false);
 	void itemSelection_SetItemFillTransparency(double t);
 	void itemSelection_SetItemLineTransparency(double t);
+	void itemSelection_SetItemFillBlend(int t);
+	void itemSelection_SetItemLineBlend(int t);
+	void itemSelection_SetLineGradient(VGradient& newGradient, Selection* customSelection=0);
+	void itemSelection_SetFillGradient(VGradient& newGradient, Selection* customSelection=0);
+	void itemSelection_ApplyImageEffects(ScImageEffectList& newEffectList, Selection* customSelection=0);
 	void itemSelection_FlipH();
 	void itemSelection_FlipV();
 	void itemSelection_DoHyphenate();
 	void itemSelection_DoDeHyphenate();
 	void itemSelection_SendToLayer(int layerNumber);
-	void itemSelection_SetParagraphStyle(int s);
 	void itemSelection_SetImageOffset(double x, double y, Selection* customSelection=0);
 	void itemSelection_SetImageScale(double x, double y, Selection* customSelection=0);
 	void itemSelection_SetImageScaleAndOffset(double ox, double oy, double sx, double sy, Selection* customSelection=0);
-	
+	void itemSelection_AlignLeftOut(AlignTo currAlignTo, double guidePosition);
+	void itemSelection_AlignRightOut(AlignTo currAlignTo, double guidePosition);
+	void itemSelection_AlignBottomIn(AlignTo currAlignTo, double guidePosition);
+	void itemSelection_AlignRightIn(AlignTo currAlignTo, double guidePosition);
+	void itemSelection_AlignBottomOut(AlignTo currAlignTo, double guidePosition);
+	void itemSelection_AlignCenterHor(AlignTo currAlignTo, double guidePosition);
+	void itemSelection_AlignLeftIn(AlignTo currAlignTo, double guidePosition);
+	void itemSelection_AlignCenterVer(AlignTo currAlignTo, double guidePosition);
+	void itemSelection_AlignTopOut(AlignTo currAlignTo, double guidePosition);
+	void itemSelection_AlignTopIn(AlignTo currAlignTo, double guidePosition);
+	void itemSelection_DistributeDistH(bool usingDistance=false, double distance=0.0);
+	void itemSelection_DistributeAcrossPage(bool useMargins=false);
+	void itemSelection_DistributeRight();
+	void itemSelection_DistributeBottom();
+	void itemSelection_DistributeCenterH();
+	void itemSelection_DistributeDistV(bool usingDistance=false, double distance=0.0);
+	void itemSelection_DistributeDownPage(bool useMargins=false);
+	void itemSelection_DistributeLeft();
+	void itemSelection_DistributeCenterV();
+	void itemSelection_DistributeTop();
+	void itemSelection_MultipleDuplicate(ItemMultipleDuplicateData&);
+	void itemSelection_UniteItems(Selection* customSelection=0);
+	void itemSelection_SplitItems(Selection* customSelection=0);
+	/**
+	 * Adjust an image frame's size to fit the size of the image in it
+	 */
+	void itemSelection_adjustFrametoImageSize(Selection* customSelection=0);
+	//! @brief startArrowID or endArrowID of -1 mean not applying a selection at this point.
+	void itemSelection_ApplyArrowHead(int startArrowID=-1, int endArrowID=-1, Selection* customSelection=0);
+
 	void ItemPen(QString farbe);
 	void ItemPenShade(int sha);
 	void ItemBrush(QString farbe);
 	void ItemBrushShade(int sha);
 	void ItemGradFill(int typ);
-	
+	void ItemPatternFill(QString pattern);
+	void ItemPatternProps(double scaleX, double scaleY, double offsetX, double offsetY, double rotation);
+
 	void updatePic();
 	void updatePict(QString name);
 	void removePict(QString name);
